% G_FEAT - Feature extraction
% 
% Performs the feature extraction for the database stored in [G_OUTPUT]/mitdb.
% The feature sets will be stored in [G_OUTPUT]/feat/*.mat.
% 
% Copyright (C) 2018 ELTE IK NumAnal, GNU GPL 3

function g_feat
    g_mkdir('feat');

    %% settings
    pre_samples = 100;
    post_samples = 200;
    optim_window_size = 5 * 60;
    pole_mult = [4 2 2];
    
    %% features
    features1_mts = cell(1, 2);
    features2_mts = cell(1, 2);
    features1_poles = cell(1, 2);
    features2_poles = [];
    features_rr = [];
    
    labels = [];
    poles1 = cell(1, 2);
    poles2 = cell(1);
    
    %% records
    fprintf('Segment heartbeats, optimize poles...\n');
    reclist = g_data_load_reclist;
    for rec = 1:length(reclist)
        fprintf('  %s\n', reclist{rec});

        %% data and attributes
        [signal, fa, loc, cls] = g_data_load(reclist{rec}, '01_filter', '01_16');
        optim_window = ceil(optim_window_size * fa);

        %% segment beats
        [samples_leads, beatmask] = g_segment_beats(signal, loc, pre_samples, post_samples, true);
        n_samples = sum(beatmask);
        n_optim_samples = find(loc(beatmask) < optim_window, 1, 'last');
        optim_samples_leads = samples_leads(1:n_optim_samples, :, :);

        %% RR features
        rr_feat = g_feat_rr(loc, fa, false);
        rr_feat = rr_feat(beatmask, :);

        %% poles
        optim_samples1 = optim_samples_leads(:, :, 1);
        poles_init = g_feat_rat_poles_init(optim_samples1);
        
        p1 = cell(1, 2);
        for lead = 1:2
            optim_samples = optim_samples_leads(:, :, lead);
            p1{lead} = g_feat_rat_poles_optim(optim_samples, pole_mult, poles_init, 1e-6);
            poles1{lead} = [poles1{lead}; p1{lead}];
        end
        optim_samples_all = [optim_samples_leads(:, :, 1); optim_samples_leads(:, :, 2)];
        p2 = g_feat_rat_poles_optim(optim_samples_all, pole_mult, poles_init, 1e-6);
        poles2{1} = [poles2{1}; p2];

        %% process leads
        for lead = 1:2
            samples = samples_leads(:, :, lead);

            %% rat features
            rat_feat1 = g_feat_rat_mts(samples, p1{lead}, pole_mult);
            rat_feat2 = g_feat_rat_mts(samples, p2, pole_mult);

            %% features
            features1_mts{lead} = [features1_mts{lead}; rat_feat1];
            features2_mts{lead} = [features2_mts{lead}; rat_feat2];
            features1_poles{lead} = [features1_poles{lead}; ones(n_samples, 1)*p1{lead}]; 
        end
        features2_poles = [features2_poles; ones(n_samples, 1)*p2];
        features_rr = [features_rr; rr_feat];
        labels = [labels cls(beatmask)];
    end

    %% save
    fprintf('Generate train/test sets...\n');
    [train_beats, test_beats, train_classes, test_classes] = g_feat_sets(labels);
    fprintf('Save features...\n');
    
    % MT422_RR_diff_I
    features = cell(1, 2);
    for lead = 1:2
        features{lead} = [features1_mts{lead}, features_rr];
    end
    g_feat_sets_save('MT422_RR_diff_I', features, labels, train_beats, test_beats, train_classes, test_classes, poles1);
    % MT422_RR_fi_diff_I
    features = cell(1, 2);
    for lead = 1:2
        features{lead} = [features1_mts{lead}, angle(features1_poles{lead}), features_rr];
    end
    g_feat_sets_save('MT422_RR_fi_diff_I', features, labels, train_beats, test_beats, train_classes, test_classes, poles1);
    % MT422_RR_r_fi_diff_I
    features = cell(1, 2);
    for lead = 1:2
        features{lead} = [features1_mts{lead}, abs(features1_poles{lead}), angle(features1_poles{lead}), features_rr];
    end
    g_feat_sets_save('MT422_RR_r_fi_diff_I', features, labels, train_beats, test_beats, train_classes, test_classes, poles1);
    % MT422_RR_same_I
    features = cell(1, 2);
    for lead = 1:2
        features{lead} = [features2_mts{lead}, features_rr];
    end
    g_feat_sets_save('MT422_RR_same_I', features, labels, train_beats, test_beats, train_classes, test_classes, poles2);
    % MT422_RR_fi_same_I
    features = cell(1, 2);
    for lead = 1:2
        features{lead} = [features2_mts{lead}, angle(features2_poles), features_rr];
    end
    g_feat_sets_save('MT422_RR_fi_same_I', features, labels, train_beats, test_beats, train_classes, test_classes, poles2);
    % MT422_RR_r_fi_same_I
    features = cell(1, 2);
    for lead = 1:2
        features{lead} = [features2_mts{lead}, abs(features2_poles), angle(features2_poles), features_rr];
    end
    g_feat_sets_save('MT422_RR_r_fi_same_I', features, labels, train_beats, test_beats, train_classes, test_classes, poles2);
    
    % MT422_RR_diff_II
    features = cell(1);
    features{1} = [features1_mts{1}, features1_mts{2}, features_rr];
    g_feat_sets_save('MT422_RR_diff_II', features, labels, train_beats, test_beats, train_classes, test_classes, poles1);
    % MT422_RR_fi_diff_II
    features = cell(1);
    features{1} = [features1_mts{1}, features1_mts{2}, angle(features1_poles{1}), angle(features1_poles{2}), features_rr];
    g_feat_sets_save('MT422_RR_fi_diff_II', features, labels, train_beats, test_beats, train_classes, test_classes, poles1);
    % MT422_RR_r_fi_diff_II
    features = cell(1);
    features{1} = [features1_mts{1}, features1_mts{2}, abs(features1_poles{1}), angle(features1_poles{1}), abs(features1_poles{2}), angle(features1_poles{2}), features_rr];
    g_feat_sets_save('MT422_RR_r_fi_diff_II', features, labels, train_beats, test_beats, train_classes, test_classes, poles1);
    % MT422_RR_same_II
    features = cell(1);
    features{1} = [features2_mts{1}, features2_mts{2}, features_rr];
    g_feat_sets_save('MT422_RR_same_II', features, labels, train_beats, test_beats, train_classes, test_classes, poles2);
    % MT422_RR_fi_same_II
    features = cell(1);
    features{1} = [features2_mts{1}, features2_mts{2}, angle(features2_poles), features_rr];
    g_feat_sets_save('MT422_RR_fi_same_II', features, labels, train_beats, test_beats, train_classes, test_classes, poles2);
    % MT422_RR_r_fi_same_II
    features = cell(1);
    features{1} = [features2_mts{1}, features2_mts{2}, abs(features2_poles), angle(features2_poles), features_rr];
    g_feat_sets_save('MT422_RR_r_fi_same_II', features, labels, train_beats, test_beats, train_classes, test_classes, poles2);
end
