% G_SIMPLEX - Nelder-Mead simplex algorithm.
% 
% Modification of simplex_* from RAIT.
%
% Copyright (C) 2018 ELTE IK NumAnal, GNU GPL 3

function poles = g_simplex(func, poles_init, eps)

% initial simplex
polenum = length(poles_init); % number of poles
n = 2*polenum; % dimension
ne = n + 1; % number of vertices of the simplex
inits = params2coords(poles_init);

% simplex:
% vertices1coord1 vertices1coord2 ... vertices1coordn function value1
% vertices2coord1 vertices2coord2 ... vertices2coordn function value2
% ...
% verticesne                      ...                 function valuene
simplex = zeros(ne,ne);
h = .1;
simplex(1,:) = [inits 0];
he          = h * ones(1,1+polenum*2);
for j = 2:ne
    simplex(j,:) = simplex(1,:);
    simplex(j,j-1) = simplex(j,j-1) + he(j-1);
end

alfa = 1; % reflection parameter
beta = 0.5; % narrowing parameter
gamma = 2; % extending parameter
if nargin<3
    eps=1e-6;
end

% values of the initial simplex

for j = 1:ne
    simplex(j,ne) = func(coords2params(simplex(j,1:n)));
end

% iteration

step = 0;
while std(simplex(:,ne)) > eps
    step = step + 1;
    
	% ordering
	simplex = sortrows(simplex,ne);
	
	% centroid
	xs = sum(simplex(1:n,1:n)) / n;
	
	% reflected point
	xr = (1 + alfa) * xs - alfa * simplex(n+1,1:n);
	fvxr = func(coords2params(xr));
	
	if fvxr < simplex(1,ne) % best function value    
        % examining the extended point
        xe = gamma * xr + (1 - gamma) * xs;
        fvxe = func(coords2params(xe));
        
        if fvxe < fvxr
            simplex(ne,:) = [xe,fvxe];
        else
            simplex(ne,:) = [xr,fvxr];
        end
	end
	
	if fvxr >= simplex(1,ne) && fvxr <= simplex(n,ne) % 2. kzepes fggvnyrtk
        simplex(ne,:) = [xr,fvxr];
	end
	
	if fvxr > simplex(n,ne)
        % examining the narrowed point
        if fvxr >= simplex(ne,ne)
            xc = beta * simplex(ne,1:n) + (1 - beta) * xs;
        else
            xc = beta * xr + (1 - beta) * xs;
        end
        fvxc = func(coords2params(xc));
        
        if fvxc < fvxr && fvxc < simplex(ne,ne)
            simplex(ne,:) = [xc,fvxc];
        else
            for k = 2:ne
                simplex(k,1:n) = (simplex(k,1:n) + simplex(1,1:n))/2;
                simplex(k,ne) = func(coords2params(simplex(k,1:n)));
            end
        end
	end
end


% Output:

simplex = sortrows(simplex,ne);
poles = coords2params(simplex(1,1:n));

end

function k = params2coords(p)
    parnum = size(p,2);
    k = zeros(1,2*parnum);

    for j = 1:parnum
        z = p(1, j);
        x = real(z);
        y = imag(z);
        r = sqrt(x^2 + y^2);
        u = x / sqrt(1 - r^2);
        v = y / sqrt(1 - r^2);
        k(1,2*j-1) = u;
        k(1,2*j) = v;
    end
end

